package com.jiayue.ipp.idp.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jiayue.ipp.common.data.entity.InverterInfo;
import com.jiayue.ipp.common.data.entity.WindTurbineInfo;
import com.jiayue.ipp.idp.service.InverterInfoService;
import com.jiayue.ipp.idp.util.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.*;


/**
 * idp_inverter_info
 *
 * @author whc
 * @date 2022-03-18 15:49:11
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/inverterinfo")
@Api(value = "inverterinfo", tags = "idp_inverter_info管理")
public class InverterInfoController {

    private final InverterInfoService inverterInfoService;

    /**
     * 分页查询
     *
     * @param page         分页对象
     * @param inverterInfo idp_inverter_info
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    public R getInverterInfoPage(Page page, InverterInfo inverterInfo) {
        return R.ok(inverterInfoService.page(page, Wrappers.query(inverterInfo)));
    }

    /**
     * 根据场站编号分页查询
     *
     * @param currentPage
     * @param pageSize
     * @param stationCode
     * @return
     */
    @ApiOperation(value = "根据场站编号分页查询", notes = "分页查询")
    @PostMapping("/getByStationCode")
    public R getByStationCode(Long currentPage, Long pageSize, String stationCode) {
        Page page = new Page(currentPage, pageSize);
        page.setMaxLimit((long) -1);
        return R.ok(inverterInfoService.page(page, inverterInfoService.getByStationCode(stationCode)));
    }

    /**
     * 通过id查询idp_inverter_info
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{id}")
    public R getById(@PathVariable("id") String id) {
        return R.ok(inverterInfoService.getById(id));
    }

    /**
     * 新增idp_inverter_info
     *
     * @param inverterInfo idp_inverter_info
     * @return R
     */
    @ApiOperation(value = "新增idp_inverter_info", notes = "新增idp_inverter_info")
    @PostMapping
    public R save(@RequestBody InverterInfo inverterInfo) {
        return R.ok(inverterInfoService.save(inverterInfo));
    }

    /**
     * 修改idp_inverter_info
     *
     * @param inverterInfo idp_inverter_info
     * @return R
     */
    @ApiOperation(value = "修改idp_inverter_info", notes = "修改idp_inverter_info")
    @PutMapping
    public R updateById(@RequestBody InverterInfo inverterInfo) {
        return R.ok(inverterInfoService.updateById(inverterInfo));
    }

    /**
     * 通过id删除idp_inverter_info
     *
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除idp_inverter_info", notes = "通过id删除idp_inverter_info")
    @DeleteMapping("/{id}")
    public R removeById(@PathVariable String id) {
        return R.ok(inverterInfoService.removeById(id));
    }

    /**
     * 根据场站编号查询 返回map格式
     *
     * @param stationCode
     * @return
     */
    @ApiOperation(value = "根据场站编号查询", notes = "分页查询")
    @PostMapping("/findByStationCode")
    public R findByStationCode(String stationCode) {
        List<InverterInfo> inverterInfoList = inverterInfoService.findByStationCode(stationCode);
        List<Map<String, String>> list = new ArrayList<>();
        for (InverterInfo e : inverterInfoList) {
            Map<String, String> map = new HashMap<>();
            map.put("label", e.getName());
            map.put("value", e.getEquipmentNo());
            list.add(map);
        }
        return R.ok(list);
    }

    @GetMapping("/all")
    public R findAll() {
        List<InverterInfo> inverterInfoList = inverterInfoService.list();
        List<Map<String, String>> list = new ArrayList<>();
        for (InverterInfo e : inverterInfoList) {
            Map<String, String> map = new HashMap<>();
            map.put("label", e.getName());
            map.put("value", e.getId());
            list.add(map);
        }
        return R.ok(list);
    }

    /**
     * 查找设备编号是否存在
     *
     * @param stationCode
     * @return
     */
    @GetMapping("/findExistEquipmentNo")
    public R findExistEquipmentNo(String stationCode,String equipmentNo) {
        QueryWrapper<InverterInfo> wrapper = new QueryWrapper<>();
        wrapper.eq("station_code", stationCode);
        wrapper.eq("equipment_no", equipmentNo);
        List<InverterInfo> inverterInfoList = inverterInfoService.list(wrapper);
        return R.ok(inverterInfoList);
    }

    @GetMapping(value = {"/exportTemplateEvent" })
    public R exportTemplateEvent(HttpServletResponse response) {
        try {
            ExcelWriter writer = ExcelUtil.getWriter(true);
            writer.renameSheet(0, "逆变器信息");
            List<String> headFieldName = Arrays.asList("场站编号","设备编号","逆变器名称","制造商","型号","安装时间(yyyy-MM-dd)","是否样板(1=>是,0=>否)","是否组串式(1=>是,0=>否)","入库时间间隔(1=>1分钟,5=>5分钟)","逆变器效率(整型数值)","额定功率(数值)","集电线路");
            List<List<String>> rowHeather = CollUtil.newArrayList();
            rowHeather.add(headFieldName);
            writer.write(rowHeather, true);
            String fileName = "逆变器信息"+ ".xlsx";
            response.setContentType("application/x-msdownload;charset=utf-8");
            response.setHeader("Content-disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8"));
            ServletOutputStream out = null;
            out = response.getOutputStream();
            writer.flush(out, true);
            // 关闭writer，释放内存
            writer.close();
            //此处记得关闭输出Servlet流
            IoUtil.close(out);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return R.ok();
    }

    /**
     * 上传升级文件
     */
    @PostMapping(value = "/uploadNbqTemplate")
    public R uploadNbqTemplate(@RequestParam(value="file") MultipartFile file){
        try (InputStream inputStream = file.getInputStream()) {
            ExcelReader excelReader = ExcelUtil.getReader(inputStream, 0);
            List<List<Object>> read = excelReader.read(1, excelReader.getRowCount());
            List<InverterInfo> list = new ArrayList<>();
            for (List<Object> objects : read) {
                InverterInfo inverterInfo = new InverterInfo();
                // 读取某行第一列数据
                inverterInfo.setStationCode(objects.get(0).toString());
                // 读取某行第二列数据
                inverterInfo.setEquipmentNo(objects.get(1).toString());
                inverterInfo.setName(objects.get(2).toString());
                inverterInfo.setManufacturer(objects.get(3).toString());
                inverterInfo.setModelNumber(objects.get(4).toString());
                inverterInfo.setInstallationTime(DateUtil.parse(objects.get(5).toString(),"yyyy-MM-dd"));
                inverterInfo.setSample(objects.get(6).toString());
                inverterInfo.setGroupSeries(objects.get(7).toString());
                inverterInfo.setTimeInterval(Integer.parseInt(objects.get(8).toString()));
                inverterInfo.setEfficiency(new BigDecimal(objects.get(9).toString()));
                inverterInfo.setCapacity(new BigDecimal(objects.get(10).toString()));
                inverterInfo.setCollectorCircuit(objects.get(11).toString());
                list.add(inverterInfo);
            }
            inverterInfoService.saveBatch(list);
        } catch (Exception e) {
            e.printStackTrace();
//            log.error("导入风机信息文件失败",e);
            return R.failed("导入逆变器信息文件失败");
        }
        return R.ok();
    }
}
